package com.xydl.cac.thermal;

import com.hcsdk.HCModule;
import com.xydl.cac.entity.NSensor;
import com.xydl.cac.entity.ThermalConfig;
import com.xydl.cac.model.StaticVariable;
import com.xydl.cac.service.DataService;
import com.xydl.cac.util.DateUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.PreDestroy;
import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

@Service
@Slf4j
public class HCThermalService {

    @Resource
    DataService dataService;
    @Resource
    HCRemoteConfigCallBack _callBack;

    ThermalConfig _config;
    int shutdown = 0;

    @PreDestroy
    private void preDestroy() {
        shutdown = 1;
    }

    public void collectAndSave(List<NSensor> list, ThermalConfig config) {
        if (shutdown == 1) {
            return;
        }
        _config = config;
        try {
            HCModule.init();
            NSensor sensor = list.get(0);
            boolean r = HCModule.login(sensor.getIp(), sensor.getPort().shortValue(), sensor.getUsername(), sensor.getPasswd());
            if (r) {
                log.info("海康热成像登入成功" + sensor.getIp() + ":" + sensor.getPort()
                        + ", 用户名:" + sensor.getUsername() + ", 密码:" + sensor.getPasswd());
            } else {
                return;
            }
            for (NSensor item : list) {
                _callBack.done = false;
                _callBack.item = item;
                _callBack.time = DateUtil.format(new Date());
                _callBack.service = this;
                r = HCModule.startRemoteConfig(_callBack, 2, item.getTmId());
                if (r) {
                    int sec = 0;
                    while (shutdown == 0 && sec < 20 && !_callBack.done) {
                        try {
                            Thread.sleep(1000);
                            sec++;
                        } catch (Exception ignore) {
                        }
                    }
                    HCModule.stopRemoteConfig();
                }
            }
            HCModule.logout();
        } catch (Exception ex) {
            log.error("海康热成像测温采集异常, " + ex.getMessage(), ex);
        } finally {
            HCModule.cleanup();
        }
    }

    @Async
    public void save(String tableName, Integer devId,
                     float max, float min, float aver, String time) {
        if (StringUtils.isNotBlank(_config.getMaxtemp())) {
            String value = String.valueOf(max);
            dataService.insertData(tableName, devId, time, _config.getMaxtemp(), value);
            // 更新最新数据缓存
            StaticVariable.updateLastData(devId, _config.getMaxtemp(), value, time);
        }
        if (StringUtils.isNotBlank(_config.getMintemp())) {
            String value = String.valueOf(min);
            dataService.insertData(tableName, devId, time, _config.getMintemp(), value);
            // 更新最新数据缓存
            StaticVariable.updateLastData(devId, _config.getMintemp(), value, time);
        }
        if (StringUtils.isNotBlank(_config.getAvertemp())) {
            String value = String.valueOf(aver);
            dataService.insertData(tableName, devId, time, _config.getAvertemp(), value);
            // 更新最新数据缓存
            StaticVariable.updateLastData(devId, _config.getAvertemp(), value, time);
        }
    }
}
